/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>

#include "ActionFO.h"
#include "ActionCommand.h"
#include "Face.h"
#include "GC.h"
#include "KeyAction.h"
#include "KeyEntry.h"
#include "Scene/SceneMediator.h"
#include "Sound.h"
#include "Window.h"

ActionFO::ActionFO()
{
    face = new Face();
}

ActionFO::~ActionFO()
{
}

void ActionFO::init(SceneMediator* mediator)
{
    mediator->addAction(this);
    mediator->addCollision(this);
    mediator->addFace(face);

    xbound1_ = xbound1 - width;
    ybound1_ = ybound1 - height;
    xbound2_ = xbound2 + mediator->getWindowWidth();
    ybound2_ = ybound2 + mediator->getWindowHeight();

    list<KeyAction*>::iterator i = keyAction.begin();
    while ( i != keyAction.end() ) {
        (*i)->setTarget(this);
        (*i)->entry(mediator);
        i++;
    }
}

void ActionFO::refresh()
{
    face->inactive = false;
    face->waitingForSurface = false;
    face->destroy = false;
}

bool ActionFO::action(SceneMediator* mediator) {
    actionCommand->action(mediator, this);

    if ( life <= 0 ) {
        destroyedCommand->action(mediator, this);
        mediator->removeCollision(this);
        mediator->removeFace(face);
        list<KeyAction*>::iterator i = keyAction.begin();
        while ( i != keyAction.end() ) {
            mediator->removeKeyListener((*i)->getKeyName());
            (*i)->destroy();
            i = keyAction.erase(i);
        }
        destroy();
        return false;
    }
    else if ( x < xbound1_
        || y < ybound1_
        || x > xbound2_
        || y > ybound2_
    ) {
        mediator->removeCollision(this);
        mediator->removeFace(face);
        list<KeyAction*>::iterator i = keyAction.begin();
        while ( i != keyAction.end() ) {
            mediator->removeKeyListener((*i)->getKeyName());
            (*i)->destroy();
            i = keyAction.erase(i);
        }
        destroy();
        return false;
    }

    x += dx;
    y += dy;
    dx /= slowdownRate;
    dy /= slowdownRate;

    face->setDistRect(x+offsetFaceX, y+offsetFaceY);
    return true;
}

bool ActionFO::collision(SceneMediator* mediator, CollisionListener* fo)
{
    reactionCommand->action(mediator, this);
    if ( damageAmount > 0 ) { fo->damaged(mediator, this, damageAmount); }
    if ( powerUpAmount > 0 ) { fo->powered(mediator, this, powerUpAmount); }

    return true;
}

void ActionFO::damaged(SceneMediator* mediator, CollisionListener* listener, int damage)
{
    damagedCommand->action(mediator, this);
    life -= damage;
}

void ActionFO::powered(SceneMediator* mediator, CollisionListener* listener, int amount)
{
    powerUpCommand->action(mediator, this);
}

void ActionFO::destroy()
{
    if ( actionCommand != NULL ) { actionCommand->destroy(); }
    if ( reactionCommand != NULL ) { reactionCommand->destroy(); }
    if ( damagedCommand != NULL ) { damagedCommand->destroy(); }
    if ( destroyedCommand != NULL ) { destroyedCommand->destroy(); }
    if ( powerUpCommand != NULL ) { powerUpCommand->destroy(); }
    GC<ActionFO>::destroy(this);
}

void ActionFO::destroy(SceneMediator* mediator)
{
    mediator->removeCollision(this);
    mediator->removeFace(face);
    mediator->removeAction(this);

    list<KeyAction*>::iterator i = keyAction.begin();
    while ( i != keyAction.end() ) {
        mediator->removeKeyListener((*i)->getKeyName());
        (*i)->destroy();
        i = keyAction.erase(i);
    }
    keyAction.clear();

    this->destroy();
}

ActionFO* ActionFO::create()
{
    ActionFO* obj = GC<ActionFO>::getInstance();
    obj->refresh();
    return obj;
}

ActionFO* ActionFO::clone()
{
    ActionFO* obj = create();
    obj->x = x;
    obj->y = y;
    obj->width = width;
    obj->height = height;

    obj->face = new Face();
    obj->face->setSurface(face->surface);
    obj->face->initFrame(face->frameWidth, face->frameHeight, face->frameInterval);
    obj->face->setDistRect(x,y);
    obj->face->layer = face->layer;

    obj->setSpeed(dx,dy);
    obj->setBound(xbound1, ybound1, xbound2, ybound2);
    obj->life = life;
    obj->actionCommand = actionCommand->clone();
    obj->reactionCommand = reactionCommand->clone();
    obj->damagedCommand = damagedCommand->clone();
    obj->destroyedCommand = destroyedCommand->clone();
    obj->powerUpCommand = powerUpCommand->clone();
    obj->offsetFaceX = offsetFaceX;
    obj->offsetFaceY = offsetFaceY;
    obj->slowdownRate = slowdownRate;
    obj->damageAmount = damageAmount;
    obj->impactResilience = impactResilience;
    obj->setGroup(getGroup());

    list<KeyAction*>::iterator i = keyAction.begin();
    while ( i != keyAction.end() ) {
        KeyAction* k = (*i)->clone();
        k->setTarget(obj);
        obj->keyAction.push_back(k);
        i++;
    }

    return obj;
}

void ActionFO::setBound(int x1, int y1, int x2, int y2)
{
    xbound1 = x1;
    ybound1 = y1;
    xbound2 = x2;
    ybound2 = y2;
}

void ActionFO::setSpeed(double dx, double dy)
{
    this->dx = dx;
    this->dy = dy;
}

void ActionFO::setAction(ActionCommand* command)
{
    actionCommand = command->clone();
}

void ActionFO::setReaction(ActionCommand* command)
{
    reactionCommand = command->clone();
}

void ActionFO::setDamaged(ActionCommand* command)
{
    damagedCommand = command->clone();
}

void ActionFO::setPowered(ActionCommand* command)
{
    powerUpCommand = command->clone();
}

void ActionFO::setDestroyed(ActionCommand* command)
{
    destroyedCommand = command->clone();
}

void ActionFO::setSurface(SDL_Surface* image, int frameWidth, int frameHeight, int frameInterval)
{
    face->setSurface(image);
    face->initFrame(frameWidth, frameHeight, frameInterval);
    face->setDistRect(x,y);
}

void ActionFO::setOffset(int x, int y)
{
    offsetFaceX = x;
    offsetFaceY = y;
}

void ActionFO::setFaceDirection(int direction)
{
    face->frameRowIndex = direction;
    face->frameCollumnIndex = 0;
    face->frameIntervalCount = face->frameInterval;
    face->setSrcRect( 
        0,
        face->frameHeight * face->frameRowIndex,
        face->frameWidth,
        face->frameHeight
    );
}

void ActionFO::setSlowdownRate(double rate)
{
    slowdownRate = rate;
}

void ActionFO::setFaceAlpha(Uint8 alpha)
{
    face->alpha = alpha;
}

void ActionFO::addKeyAction(KeyEntry* keyEntry)
{
    keyAction.push_back( new KeyAction(this, keyEntry) );
}

void ActionFO::setDamageAmount(int amount)
{
    damageAmount = amount;
}

void ActionFO::setLayer(int layer)
{
    if ( 0 <= layer && layer < Window::LAYERMAX ) {
        face->layer = layer;
    }
}
