/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __ACTIONFO_H__INCLUDED__
#define __ACTIONFO_H__INCLUDED__
#include "FlyingObject.h"
#include "Scene/ActionListener.h"
#include "Scene/CollisionListener.h"

#include <list>
#include <SDL/SDL.h>

class ActionCommand;
class Face;
class SceneMediator;
class KeyAction;
class KeyEntry;

/**
 * このオブジェクトは移動、アニメーションなどの振るまいを
 * アクションシーケンス使って決定する。
 */
class ActionFO :
public FlyingObject, public ActionListener, public CollisionListener
{
private:
    int xbound1_, xbound2_, ybound1_, ybound2_; // 移動可能範囲(諸計算済み)
    void refresh();

    ActionCommand* actionCommand;
    ActionCommand* reactionCommand;
    ActionCommand* damagedCommand;
    ActionCommand* destroyedCommand;
    ActionCommand* powerUpCommand;
    Face* face;
    double slowdownRate;
    int xbound1, xbound2, ybound1, ybound2; // 移動可能範囲(補正値)
    int offsetFaceX, offsetFaceY;
    int damageAmount;
    int powerUpAmount;
    list<KeyAction*> keyAction;

public:
    ActionFO();
    virtual ~ActionFO();
    virtual bool action(SceneMediator* mediator);
    virtual bool collision(SceneMediator* mediator, CollisionListener* fo);
    virtual void damaged(SceneMediator* mediator, CollisionListener* fo, int damage);
    virtual void powered(SceneMediator* mediator, CollisionListener* fo, int amoount);
    virtual void destroy();
    virtual void destroy(SceneMediator* mediator);
    virtual ActionFO* clone();
    static ActionFO* create();
    void setBound(int x1, int y1, int x2, int y2);
    void setOffset(int x, int y);
    void setDamageAmount(int x);
    void setPowerUpAmount(int x);
    void setSpeed(double dx, double dy);
    void setSlowdownRate(double rate);
    void init(SceneMediator* mediator);

    void setAction(ActionCommand* action);
    void setReaction(ActionCommand* action);
    void setDamaged(ActionCommand* action);
    void setPowered(ActionCommand* action);
    void setDestroyed(ActionCommand* action);

    void addKeyAction(KeyEntry* keyEntry);
    void setSurface(SDL_Surface* image, int fwidth, int fheight, int fint);
    void setFaceDirection(int direction);
    void setFaceAlpha(Uint8 alpha);
    void setLayer(int layer);
};

#endif
