/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include "BGScroll.h"
#include "Face.h"
#include "GC.h"
#include "Library.h"
#include "Scene/SceneMediator.h"

BGScroll::BGScroll()
{
}

BGScroll::~BGScroll()
{
}

BGScroll* BGScroll::create(const char* file, double dx, double dy, int time)
{
    BGScroll* bgScroll = GC<BGScroll>::getInstance();
    bgScroll->surface = Library::loadImage(file);
    bgScroll->tileWidth = bgScroll->surface->w;
    bgScroll->tileHeight = bgScroll->surface->h;
    bgScroll->offsetX = 0;
    bgScroll->offsetY = 0;
    bgScroll->dx = dx;
    bgScroll->dy = dy;
    bgScroll->timeLeft = time;

    return bgScroll;
}
void BGScroll::init(SceneMediator* mediator)
{
    faceSizeX = mediator->getWindowWidth() / surface->w + 2;
    faceSizeY = mediator->getWindowHeight() / surface->h + 2;
    faceSize = faceSizeX * faceSizeY;

    face = new Face[faceSize]();

    for ( int y = 0; y < faceSizeY; y++ ) {
        for ( int x = 0; x < faceSizeX; x++ ) {
            Face* f = &face[faceSizeX*y + x];
            f->setSurface(surface);
            f->initFrame();
            f->setDistRect(x*tileWidth, y*tileHeight, tileWidth, tileHeight);
            f->layer = 0;
            mediator->addFace(f);
        }
    }

    mediator->addAction(this);
}

void BGScroll::destroy()
{
    delete[] face;
    GC<BGScroll>::destroy(this);
}

void BGScroll::destroy(SceneMediator* mediator)
{
    mediator->removeAction(this);
    for ( int i=0; i < faceSize; i++ ) {
        mediator->removeFace(&face[i]);
    }
    destroy();
}

bool BGScroll::action(SceneMediator* mediator)
{
    if ( timeLeft-- <= 0 ) {
        for ( int i=0; i<faceSize; i++ ) { mediator->removeFace(&face[i]); }
        destroy();
        return false;
    }

    offsetX -= dx;
    if ( offsetX >= tileWidth ) offsetX = 0;
    else if ( offsetX < 0 ) offsetX += tileWidth;
 
    offsetY -= dy;
    if ( offsetY >= tileHeight ) offsetY = 0;
    else if ( offsetY < 0 ) offsetY += tileHeight;

    for ( int y = 0; y < faceSizeY; y++ ) {
        for ( int x = 0; x < faceSizeX; x++ ) {
            face[faceSizeX*y + x].setDistRect(
                (Sint32)(x*tileWidth+offsetX-tileWidth),
                (Sint32)(y*tileHeight+offsetY-tileHeight),
                tileWidth,
                tileHeight
            );
        }
    }
    return true;
}

BGScroll* BGScroll::clone()
{
    BGScroll* bgScroll = GC<BGScroll>::getInstance();
    bgScroll->surface = surface;
    bgScroll->tileWidth = bgScroll->surface->w;
    bgScroll->tileHeight = bgScroll->surface->h;
    bgScroll->offsetX = 0;
    bgScroll->offsetY = 0;
    bgScroll->dx = dx;
    bgScroll->dy = dy;
    bgScroll->timeLeft = timeLeft;

    return bgScroll;
}
