/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __FACE_H__INCLUDED__
#define __FACE_H__INCLUDED__
#include <SDL/SDL.h>
#include <math.h>
#include "List.h"

class Face
{
public:
    SDL_Rect srcRect;
    SDL_Rect distRect;

    int frameWidth;
    int frameHeight;
    int frameRowIndex;
    int frameCollumnIndex;
    int frameRowMax;
    int frameCollumnMax;
    int frameInterval;
    int frameIntervalCount;

    Uint8 alpha;
    Uint32 alphaFlags;

    int layer;

    SDL_Surface* surface;
    bool waitingForSurface;
    bool inactive;
    bool destroy;

    Face();

private:
    List<Face>::Item* listIndex;
public:
    inline List<Face>::Item* getListIndex()
    {
        return listIndex;
    }

    void setListIndex(List<Face>::Item* listIndex)
    {
        this->listIndex = listIndex;
    }

    inline void setSurface(SDL_Surface* s)
    {
        surface = s;
        setSrcRect(0, 0, s->w, s->h);
        distRect.w = s->w;
        distRect.h = s->h;
    }

    inline void setSrcRect(Sint16 x, Sint16 y, Uint16 w, Uint16 h)
    {
        srcRect.x = x;
        srcRect.y = y;
        srcRect.w = w;
        srcRect.h = h;
    }

    inline void setDistRect(Sint16 x, Sint16 y, Uint16 w, Uint16 h)
    {
        distRect.x = x;
        distRect.y = y;
        distRect.w = w;
        distRect.h = h;
    }

    inline void setDistRect(Sint16 x, Sint16 y)
    {
        distRect.x = x;
        distRect.y = y;
        distRect.w = surface->w;
        distRect.h = surface->h;
    }

    inline void setDistRect(int x, int y)
    {
        setDistRect((Sint16)x, (Sint16)y);
    }

    inline void setDistRect(double x, double y)
    {
        setDistRect((Sint16)rint(x), (Sint16)rint(y));
    }

    inline void nextFrame()
    {
        if ( frameInterval < 0 || frameIntervalCount-- > 0 ) return;
        frameIntervalCount = frameInterval;
        frameCollumnIndex = (frameCollumnIndex + 1) % frameCollumnMax;
        setSrcRect(
            frameWidth * frameCollumnIndex,
            frameHeight * frameRowIndex,
            frameWidth,
            frameHeight
        );
    }


    inline void initFrame()
    {
        frameWidth = 0;
        frameHeight = 0;
        frameRowIndex = 0;
        frameCollumnIndex = 0;
        frameRowMax = 0;
        frameCollumnMax = 0;
        frameInterval = -1;
        frameIntervalCount = 0;
    }

    inline void initFrame(int width, int height, int interval)
    {
        frameWidth = width;
        frameHeight = height;
        frameRowIndex = 0;
        frameCollumnIndex = 0;
        if ( height > 0 ) { frameRowMax = surface->h / height; }
        else { frameRowMax = 0; }
        if ( width > 0 ) { frameCollumnMax = surface->w / width; }
        else { frameCollumnMax = 0; }
        frameInterval = interval;
        frameIntervalCount = interval;

        setSrcRect( 0, 0, frameWidth, frameHeight );
    }

};
#endif

