/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <math.h>

#include "Face.h"
#include "GC.h"
#include "Label.h"
#include "Library.h"
#include "Scene/SceneMediator.h"

Label::Label()
{
    face = new Face();
}

Label::~Label()
{
    delete face;
}

Label* Label::createLabel(SDL_Surface* surface, double x, double y, double dx, double dy, int timer, int extTimer, int effect)
{
    Label* label = GC<Label>::getInstance();
    label->x = x;
    label->y = y;
    label->face->setSurface(surface);
    label->face->initFrame();

    label->face->setDistRect(x,y);

    label->dx = dx;
    label->dy = dy;
    label->dalpha = 0;
    label->alpha = 255;

    label->setEffect(effect, timer);

    label->timerCount = timer;
    label->extraTimerCount = extTimer;

    return label;
}

Label* Label::createLabel(const char* text, double x, double y, double dx, double dy, int timer, int extTimer, int effect)
{
    return createLabel(Library::createString(text), x, y, dx, dy, timer, extTimer, effect);
}

Label* Label::createLabel(const char* text, SDL_Color* color, double x, double y, double dx, double dy, int timer, int extTimer, int effect)
{
    return createLabel(Library::createString(text, color), x, y, dx, dy, timer, extTimer, effect);
}

void Label::setEffect(int effect, int timer)
{
    switch(effect) {
        case 1:
        {
            dalpha = 255.0 / timer;
            alpha = 0;
            break;
        }
        case 2:
        {
            dalpha = -255.0 / timer;
            alpha = 255;
            break;
        }
    }
}

bool Label::action(SceneMediator* mediator)
{
    if( timerCount-- <= 0 ) {
        if ( extraTimerCount-- <= 0 ) {
            mediator->removeFace(face);
            destroy();
            return false;
        }
    }
    x += dx;
    y += dy;
    alpha += dalpha;
    if ( alpha < 0 ) { alpha = 0; } else if ( alpha > 255 ) { alpha = 255; }
    face->setDistRect(x, y);
    face->alpha = (Uint8)rint(alpha);
    return true;
}

void Label::destroy()
{
    GC<Label>::destroy(this);
}

void Label::destroy(SceneMediator* mediator)
{
    mediator->removeFace(face);
    mediator->removeAction(this);
    destroy();
}

Label* Label::clone()
{
    Label* label = GC<Label>::getInstance();
    label->x = x;
    label->y = y;
    label->face->setSurface(face->surface);
    label->face->initFrame();

    label->face->setDistRect(x,y);
    label->face->alpha = (Uint8)alpha;

    label->dx = dx;
    label->dy = dy;
    label->dalpha = dalpha;
    label->alpha = alpha;

    label->timerCount = timerCount;
    label->extraTimerCount = extraTimerCount;

    return label;
}

void Label::init(SceneMediator* mediator)
{
    mediator->addAction(this);
    mediator->addFace(face);
}
