/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <iostream>
#include <string>
#include <map>
#include <stdlib.h>

#include "Font.h"
#include "Library.h"

using namespace std;

/*
 * ライブラリ初期化
 */
Uint32 SDL_FLAGS  = SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER;
Uint32 VIDEO_FLAGS = SDL_HWSURFACE|SDL_SRCALPHA|SDL_ANYFORMAT;
Uint32 SURFACE_FLAGS = SDL_HWSURFACE|SDL_SRCALPHA;
int BPP = 32;

map<string, SDLKey> Library::keyboardMap;

bool Library::initialized = false;
void Library::init()
{
    if ( initialized ) {
        cout << "Library::init(): invalid call." << endl;
        return;
    }

    // SDL初期化
    if ( SDL_Init(SDL_FLAGS) == -1 ) {
        cerr << "SDL_Init: " << SDL_GetError() << endl;
        exit(-1);
    }

    // マウスカーソルを表示に設定
    SDL_ShowCursor(SDL_DISABLE);

    // SDL_ttfの初期化
    if ( TTF_Init() == -1 ) {
        cerr << "TTF_Init: " << TTF_GetError() << endl;
        exit(-1);
    }

    if ( SDL_WasInit(SDL_INIT_AUDIO) == 0 ) {
        SDL_InitSubSystem(SDL_INIT_AUDIO);
    }

    // SDL_mixerの初期化
    if ( Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 256) == -1 ) {
        cerr << "Mix_OpenAudio: " << Mix_GetError() << endl;
        exit(-1);
    }
    Mix_AllocateChannels(3);

    // キーボードマッピング
    keyboardMap["上"] = SDLK_UP;
    keyboardMap["下"] = SDLK_DOWN;
    keyboardMap["左"] = SDLK_LEFT;
    keyboardMap["右"] = SDLK_RIGHT;
    keyboardMap["スペース"] = SDLK_SPACE;
    keyboardMap["改行"] = SDLK_RETURN;
    keyboardMap["バックスペース"] = SDLK_BACKSPACE;
    keyboardMap["タブ"] = SDLK_TAB;
    keyboardMap["エスケープ"] = SDLK_ESCAPE;
    keyboardMap["ダラー"] = SDLK_DOLLAR;
    keyboardMap["アンパーサンド"] = SDLK_AMPERSAND;
    keyboardMap["左かっこ"] = SDLK_LEFTPAREN;
    keyboardMap["右かっこ"] = SDLK_RIGHTPAREN;
    keyboardMap["アスタリスク"] = SDLK_ASTERISK;
    keyboardMap["プラス"] = SDLK_PLUS;
    keyboardMap["カンマ"] = SDLK_COMMA;
    keyboardMap["マイナス"] = SDLK_MINUS;
    keyboardMap["ピリオド"] = SDLK_PERIOD;
    keyboardMap["スラッシュ"] = SDLK_SLASH;
    keyboardMap["０"] = SDLK_0;
    keyboardMap["１"] = SDLK_1;
    keyboardMap["２"] = SDLK_2;
    keyboardMap["３"] = SDLK_3;
    keyboardMap["４"] = SDLK_4;
    keyboardMap["５"] = SDLK_5;
    keyboardMap["６"] = SDLK_6;
    keyboardMap["７"] = SDLK_7;
    keyboardMap["８"] = SDLK_8;
    keyboardMap["９"] = SDLK_9;
    keyboardMap["コロン"] = SDLK_COLON;
    keyboardMap["セミコロン"] = SDLK_SEMICOLON;
    keyboardMap["小なり"] = SDLK_LESS;
    keyboardMap["等号"] = SDLK_EQUALS;
    keyboardMap["大なり"] = SDLK_GREATER;
    keyboardMap["クエスチョンマーク"] = SDLK_QUESTION;
    keyboardMap["＠"] = SDLK_AT;
    keyboardMap["左ブラケット"] = SDLK_LEFTBRACKET;
    keyboardMap["右ブラケット"] = SDLK_RIGHTBRACKET;
    keyboardMap["キャレット"] = SDLK_CARET;
    keyboardMap["アンダースコア"] = SDLK_UNDERSCORE;
    keyboardMap["バッククォート"] = SDLK_BACKQUOTE;
    keyboardMap["バックスラッシュ"] = SDLK_BACKSLASH;
    keyboardMap["a"] = SDLK_a;
    keyboardMap["b"] = SDLK_b;
    keyboardMap["c"] = SDLK_c;
    keyboardMap["d"] = SDLK_d;
    keyboardMap["e"] = SDLK_e;
    keyboardMap["f"] = SDLK_f;
    keyboardMap["g"] = SDLK_g;
    keyboardMap["h"] = SDLK_h;
    keyboardMap["i"] = SDLK_i;
    keyboardMap["j"] = SDLK_j;
    keyboardMap["k"] = SDLK_k;
    keyboardMap["l"] = SDLK_l;
    keyboardMap["m"] = SDLK_m;
    keyboardMap["o"] = SDLK_o;
    keyboardMap["p"] = SDLK_p;
    keyboardMap["q"] = SDLK_q;
    keyboardMap["r"] = SDLK_r;
    keyboardMap["s"] = SDLK_s;
    keyboardMap["t"] = SDLK_t;
    keyboardMap["u"] = SDLK_u;
    keyboardMap["v"] = SDLK_v;
    keyboardMap["w"] = SDLK_w;
    keyboardMap["x"] = SDLK_x;
    keyboardMap["y"] = SDLK_y;
    keyboardMap["z"] = SDLK_z;
    keyboardMap["削除"] = SDLK_DELETE;

    keyboardMap["テンキー０"] = SDLK_KP0;
    keyboardMap["テンキー１"] = SDLK_KP1;
    keyboardMap["テンキー２"] = SDLK_KP2;
    keyboardMap["テンキー３"] = SDLK_KP3;
    keyboardMap["テンキー４"] = SDLK_KP4;
    keyboardMap["テンキー５"] = SDLK_KP5;
    keyboardMap["テンキー６"] = SDLK_KP6;
    keyboardMap["テンキー７"] = SDLK_KP7;
    keyboardMap["テンキー８"] = SDLK_KP8;
    keyboardMap["テンキー９"] = SDLK_KP9;
    keyboardMap["テンキーピリオド"] = SDLK_KP_PERIOD;
    keyboardMap["テンキー割り算"] = SDLK_KP_DIVIDE;
    keyboardMap["テンキーかけ算"] = SDLK_KP_MULTIPLY;
    keyboardMap["テンキーマイナス"] = SDLK_KP_MINUS;
    keyboardMap["テンキープラス"] = SDLK_KP_PLUS;
    keyboardMap["テンキーエンター"] = SDLK_KP_ENTER;
    keyboardMap["テンキーイコール"] = SDLK_KP_EQUALS;

    keyboardMap["ホーム"] = SDLK_HOME;
    keyboardMap["エンド"] = SDLK_END;
    keyboardMap["ページアップ"] = SDLK_PAGEUP;
    keyboardMap["ページダウン"] = SDLK_PAGEDOWN;

    keyboardMap["ファンクションキー１"] = SDLK_F1;
    keyboardMap["ファンクションキー２"] = SDLK_F2;
    keyboardMap["ファンクションキー３"] = SDLK_F3;
    keyboardMap["ファンクションキー４"] = SDLK_F4;
    keyboardMap["ファンクションキー５"] = SDLK_F5;
    keyboardMap["ファンクションキー６"] = SDLK_F6;
    keyboardMap["ファンクションキー７"] = SDLK_F7;
    keyboardMap["ファンクションキー８"] = SDLK_F8;
    keyboardMap["ファンクションキー９"] = SDLK_F9;
    keyboardMap["ファンクションキー１０"] = SDLK_F10;
    keyboardMap["ファンクションキー１１"] = SDLK_F11;
    keyboardMap["ファンクションキー１２"] = SDLK_F12;
    keyboardMap["ファンクションキー１３"] = SDLK_F13;
    keyboardMap["ファンクションキー１４"] = SDLK_F14;
    keyboardMap["ファンクションキー１５"] = SDLK_F15;

    // 初期化完了
    initialized = true;

    // 終了処理を登録
    atexit(Library::end);
}

void Library::end()
{
    if ( !initialized ) { 
        cout << "Library::end(): Invalid call." << endl;
        return;
    }
    freeImages();
    Mix_CloseAudio();
    TTF_Quit();
    SDL_Quit();
}

SDL_Surface* Library::createWindow(Uint32 width, Uint32 height)
{
    SDL_Surface* surface = SDL_SetVideoMode(width, height, BPP, VIDEO_FLAGS);
    if ( surface == NULL ) {
        cerr << "SDL_SetVideoMode: " << SDL_GetError() << endl;
        exit(-1);
    }

    return surface;
}
SDL_Surface* Library::createSurface(Uint32 width, Uint32 height)
{
    switch ( BPP ) {
        case 8: { return createSurface8(width, height); }
        case 16: { return createSurface16(width, height); }
        case 32: { return createSurface32(width, height); }
        default:
        {
            cerr << "Library::createSurface(";
            cerr << width << ", " << height << "): invalid bpp(";
            cerr << BPP << ") is specified." << endl;
            exit(-1);
        }
    }
}

SDL_Surface* Library::createSurface32(Uint32 width, Uint32 height)
{
    return SDL_CreateRGBSurface(
        SURFACE_FLAGS, 
        width, height, 32,
        0xff, 0xff00, 0xff0000, 0xff000000 
    );
}

SDL_Surface* Library::createSurface16(Uint32 width, Uint32 height)
{
    return SDL_CreateRGBSurface(
        SURFACE_FLAGS, 
        width, height, 16,
        0xf, 0x1f0, 0x3e00, 0xc000 
    );
}

SDL_Surface* Library::createSurface8(Uint32 width, Uint32 height)
{
    return SDL_CreateRGBSurface(
        SURFACE_FLAGS, 
        width, height, 8,
        0x3, 0xc, 0x70, 0x80 
    );
}

void Library::blitSurface(SDL_Surface*& src, SDL_Rect* srcrect, SDL_Surface*& dist, SDL_Rect* distrect)
{
    if ( SDL_MUSTLOCK(src) ) {
        if ( SDL_LockSurface(src) < 0 ) {
            cerr << "Library::blitSurface(): Can't lock src." << endl;
            exit(-1);
        }
    }
    if ( SDL_MUSTLOCK(dist) ) {
        if ( SDL_LockSurface(dist) < 0 ) {
            cerr << "Library::blitSurface(): Can't lock dist." << endl;
            exit(-1);
        }
    }

    int bpp = src->format->BytesPerPixel;

    int width = srcrect->w < distrect->w ? srcrect->w : distrect->w;
    int height = srcrect->h < distrect->h ? srcrect->h : distrect->h;

    SDL_PixelFormat* srcfmt = src->format;

    for ( int x = 0; x < width; x++ ) {
        for ( int y = 0; y < height; y++ ) {
            /*
            if ( srcrect->x + x >= src->w
                || srcrect->y + y >= src->h
                || distrect->x + x >= dist->w
                || distrect->y + y >= dist->h 
            ) {
                continue;
            }
            */

            Uint32 pixel;
            Uint8* p = (Uint8*)src->pixels + (srcrect->x + x)*bpp + (srcrect->y + y)*src->pitch;
            switch(bpp) {
                case 1: { pixel = *p; break; }
                case 2: { pixel = *(Uint16*)p; break; }
                case 3: {
                    if ( SDL_BYTEORDER == SDL_BIG_ENDIAN ) {
                        pixel = p[0] << 16 | p[1] << 8 | p[2];
                    }
                    else {
                        pixel = p[0] | p[1] << 8 | p[2] << 16;
                    }
                    break;
                }
                case 4: { pixel = *(Uint32*)p; break; }
            }
            Uint8 red, green, blue, alpha;
            SDL_GetRGBA(pixel, srcfmt, &red, &green, &blue, &alpha);

            int dbpp = dist->format->BytesPerPixel;
            Uint32 dpixel = SDL_MapRGBA(dist->format, red, green, blue, alpha);
            Uint8* q = (Uint8*)dist->pixels + (distrect->x + x)*dbpp + (distrect->y+y) * dist->pitch;

            switch ( dbpp ) {
                case 1: { *q = dpixel; break; }
                case 2: { *(Uint16*)q = dpixel; break; }
                case 3:
                {
                    if ( SDL_BYTEORDER == SDL_BIG_ENDIAN ) {
                        q[0] = (dpixel >> 16) & 0xff;
                        q[1] = (dpixel >>  8) & 0xff;
                        q[2] = dpixel & 0xff;
                    }
                    else {
                        q[0] = dpixel & 0xff;
                        q[1] = (dpixel >>  8) & 0xff;
                        q[2] = (dpixel >> 16) & 0xff;
                    }
                    break;
                }
                case 4: { *(Uint32*)q = dpixel; break; }
            }
        }
    }

    if ( SDL_MUSTLOCK(dist) ) { SDL_UnlockSurface(dist); }
    if ( SDL_MUSTLOCK(src) ) { SDL_UnlockSurface(src); }
    return;
}

SDL_Surface* Library::createString(const char* text, SDL_Color* color)
{
    return TTF_RenderUTF8_Solid(
        Font::getDefaultFont(),
        text,
        *color
    );
}

SDL_Surface* Library::createString(const char* text)
{
    return TTF_RenderUTF8_Solid(
        Font::getDefaultFont(),
        text,
        *(Font::getDefaultColor())
    );
}

SDL_Surface* Library::createLargeString(const char* text)
{
    return TTF_RenderUTF8_Solid(
        Font::getLargeFont(),
        text,
        *(Font::getDefaultColor())
    );
}

SDL_Surface* Library::createHugeString(const char* text)
{
    return TTF_RenderUTF8_Solid(
        Font::getHugeFont(),
        text,
        *(Font::getDefaultColor())
    );
}

map<string, SDL_Surface*> Library::images;

SDL_Surface* Library::loadImage(const string& file)
{
    map<string, SDL_Surface*>::iterator s = images.find(file);
    if ( s != images.end() ) { return (*s).second; }

    SDL_Surface* surface = IMG_Load(file.c_str());
    if ( surface == NULL ) {
        cerr << "IMG_Load(\"" << file << "\"): " << IMG_GetError() << endl;
        exit(-1);
    }

    images[file] = surface;

    return surface;
}

void Library::freeImages()
{
    map<string, SDL_Surface*>::iterator s = images.begin();
    while ( s != images.end() ) {
        SDL_FreeSurface( (*s).second );
        s++;
    }
    images.clear();
}

SDLKey Library::getKeySym(string name)
{
    if ( keyboardMap.find(name) == keyboardMap.end() ) { return SDLK_UNKNOWN; }
    return keyboardMap[name];
}
