/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __LIST_H__INCLUDED__
#define __LIST_H__INCLUDED__

#include <iostream>
#include <stdlib.h>

using namespace std;

// 双方向連結リスト(最大要素数固定、要素オブジェクト再利用)
template<class T>
class List
{
public:
    class Item
    {
    private:
        Item* next;
        Item* prev;
        T* object;
        bool live;

    public:
        Item();
        ~Item();
        inline Item* getNext() { return next; }
        inline Item* getPrevious() { return prev; }
        inline void setNext(Item* t) { next = t; }
        inline void setPrevious(Item* t) { prev = t; }
        inline T* getObject() { return object; }
        inline void setObject(T* object) { this->object = object; }
        inline bool getLive() { return live; }
        inline void setLive(bool live) { this->live = live; }
        inline void refresh()
        {
            next = NULL;
            prev = NULL;
            object = NULL;
            live = true;
        }
    };
 
private:
    Item* listItem;
    int size;
    int sizeMax;
    int index;

    inline Item* getItem()
    {
        for ( int i=index; i != index-1; i = (i+1)%sizeMax ) {
            if ( listItem[i+1].getLive() ) {
                continue;
            }
            else {
                listItem[i+1].refresh();
                return &listItem[i+1];
            }
        }
        cerr << "List::Item::getItem(): No availlable item." << endl;
        return NULL;
    }

public:
    List();
    List(int size);
    ~List();
    inline Item* erase(Item* item)
    {
        Item* prev = item->getPrevious();
        Item* next = item->getNext();
        prev->setNext(next);
        next->setPrevious(prev);
        item->setLive(false);
        size--;
        return next;
    }
    inline void add(T* object)
    {
        if ( size >= sizeMax ) {
            cerr << "List is full." << size << endl;
            exit(-1);
        }
        Item* i = getItem();
        Item* e = getEnd();
        i->setObject(object);
        i->setNext(e);
        i->setPrevious(e->getPrevious());
        e->getPrevious()->setNext(i);
        e->setPrevious(i);
        size++;
    }
    inline Item* getBegin() { return listItem[0].getNext(); }
    inline Item* getEnd() { return &listItem[0]; }
    inline int getSize() { return size; }
    inline int isEmpty() { return size==0?true:false; }
   
    inline void clear() {
        for( int i=1; i < sizeMax; i++ ) {
            listItem[i].setLive(false);
        }
        index = 0;
        size = 0;
        listItem[0].setNext(&listItem[0]);
        listItem[0].setPrevious(&listItem[0]);
    }
};

template<class T>
List<T>::Item::Item()
{
    live = false;
}

template<class T>
List<T>::Item::~Item()
{
}

template<class T>
List<T>::List()
{
    index = 0;
    sizeMax = 500;
    listItem = new Item[sizeMax+1];
    size = 0;

    listItem[0].setNext(&listItem[0]);
    listItem[0].setPrevious(&listItem[0]);
}

template<class T>
List<T>::List(int size)
{
    index = 0;
    sizeMax = size;
    listItem = new Item[sizeMax+1];
    this->size = 0;
    listItem[0].setNext(&listItem[0]);
    listItem[0].setPrevious(&listItem[0]);
}

template<class T>
List<T>::~List()
{
    delete[] listItem;
}

#endif

