/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdio.h>
#include <string>

#include <SDL/SDL.h>

#include "Face.h"
#include "Font.h"
#include "GC.h"
#include "Library.h"
#include "Monitor.h"
#include "Scene/CollisionGroup.h"
#include "Scene/CollisionListener.h"
#include "Scene/SceneMediator.h"
#include "Window.h"

using namespace std;

const string MONITOR_SWITCH("モニターのON/OFF");

Monitor::Monitor()
{
    int lineHeight = TTF_FontHeight(Font::getDefaultFont());

    char* labelTextString[] = {
        "Monitor ON",
        "FPS:",
        "Face:",
        "Action:",
        "Player:",
        "Score:",
        NULL
    };

    int i;
    for ( i = 0; labelTextString[i] != NULL; i++ ) {
        labelText[i] = new Face();
        labelText[i]->setSurface(Library::createString(labelTextString[i]));
        labelText[i]->layer = Window::LAYERMAX-1;
        valueText[i] = new Face();
        valueText[i]->layer = Window::LAYERMAX-1;
        valueText[i]->setSurface(Library::createString(" "));

        labelText[i]->setDistRect(0, lineHeight * i);
        valueText[i]->distRect.x = labelText[i]->surface->w;
        valueText[i]->distRect.y = lineHeight * i;
    }
    labelText[i] = NULL;
    valueText[i] = NULL;

    active = true;
    updateDelay = 11;
    count = updateDelay;
    monitorIndex = 0;
}

Monitor::~Monitor()
{
    for ( int i = 0; labelText[i] != NULL; i++ ) {
        delete labelText[i];
        delete valueText[i];
    }
}

void Monitor::activate(SceneMediator* mediator)
{
    if ( active ) return;
    for ( int i = 0; labelText[i] != NULL; i++ ) {
        labelText[i]->inactive = false;
        valueText[i]->inactive = false;
        mediator->addFace(labelText[i]);
        mediator->addFace(valueText[i]);
    }
    active = true;
    count = 0;
}

void Monitor::inactivate()
{
    if ( !active ) return;
    for ( int i = 0; labelText[i] != NULL; i++ ) {
        labelText[i]->inactive = true;
        valueText[i]->inactive = true;
    }
    active = false;
}

bool Monitor::action(SceneMediator* mediator)
{
    if ( !active ) return true;

    FPS_FrameCount++;

    if ( --count > 0 )  return true;
    count = updateDelay;
    switch(monitorIndex) {
        case 0: // label "Monitor ON"
            break;

        case 1: // FPS
        {
            Uint32 now = SDL_GetTicks();
            char str[140];
            double fps = (double)(FPS_FrameCount-FPS_LastFrameCount)/(double)((now-FPS_LastTick)/1000.0);
            sprintf(str, "%0.6f[fps], %05d[ms]/%03d[frame]", fps, now-FPS_LastTick, FPS_FrameCount-FPS_LastFrameCount);
            SDL_FreeSurface(valueText[1]->surface);
            valueText[1]->setSurface(Library::createString(str));

            FPS_LastFrameCount = FPS_FrameCount;
            FPS_LastTick = now;
            break;
        }

        case 2: //Face
        {
            char str[20];
            sprintf(str, "%05d", mediator->getFaceSize());
            SDL_FreeSurface(valueText[2]->surface);
            valueText[2]->setSurface(Library::createString(str));
            break;
        }

        case 3: //Action
        {
            char str[20];
            sprintf(str, "%05d", mediator->getActionSize());
            SDL_FreeSurface(valueText[3]->surface);
            valueText[3]->setSurface(Library::createString(str));
            break;
        }

        case 4: //Player
        {
            char str[100];
            CollisionListener* p = mediator->getObject(CollisionGroup::PLAYER, 0);
            if ( p == NULL ) { break; }
            sprintf(str, "(%03.1f,%03.1f)(%03.1f,%03.1f) %03d", p->x, p->y, p->dx, p->dy, p->life);
            SDL_FreeSurface(valueText[4]->surface);
            valueText[4]->setSurface(Library::createString(str));
            break;
        }

        case 5: //Score
        {
            char str[10];
            sprintf(str, "%08d", mediator->getScore());
            SDL_FreeSurface(valueText[5]->surface);
            valueText[5]->setSurface(Library::createString(str));
            break;
        }
 
    }

    monitorIndex++;
    if( labelText[monitorIndex] == NULL ) monitorIndex = 0;
    return true;
}

void Monitor::destroy()
{
    GC<Monitor>::destroy(this);
}

void Monitor::destroy(SceneMediator* mediator)
{
    mediator->removeAction(this);
    for ( int i = 0; labelText[i] != NULL; i++ ) {
        mediator->removeFace(labelText[i]);
        mediator->removeFace(valueText[i]);
    }
    mediator->removeKeyListener(MONITOR_SWITCH);
    destroy();
}

Monitor* Monitor::createMonitor()
{
    return GC<Monitor>::getInstance();
}

void Monitor::init(SceneMediator* mediator)
{
    for ( int i = 0; labelText[i] != NULL; i++ ) {
        mediator->addFace(labelText[i]);
        mediator->addFace(valueText[i]);
        labelText[i]->inactive = false;
        valueText[i]->inactive = false;
    }
    mediator->addAction(this);
    active = true;
    count = 0;

    mediator->addKeyListener(MONITOR_SWITCH, this);
    mediator->addKeyEvent(MONITOR_SWITCH, SDLK_m);
}

Monitor* Monitor::clone()
{
    return NULL;
}

bool Monitor::keyPressed(SceneMediator* mediator)
{
    if ( keypress ) { return true; }
    keypress = true;

    if ( active ) {
        inactivate();
    }
    else {
        activate(mediator);
    }

    return true;
}

bool Monitor::keyReleased(SceneMediator* mediator)
{
    keypress = false;
}
