/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>

#include "Scenario.h"
#include "Scene.h"
#include "Scene/SceneMediator.h"
#include "Window.h"
#include "common.h"

using namespace std;

Scenario::Scenario()
{
    window = new Window();
}

void Scenario::setWindow(Window* w)
{
    window = w;
}

Window* Scenario::getWindow()
{
    return window;
}

void Scenario::start()
{
    window->showWindow();
    Scene* initialScene = scenelist.front();

    while ( !scenelist.empty() ) {
        Scene* scene = scenelist.front();
        scene->getMediator()->setWindow(window);
    
        if ( scene->getInterrupted() ) {
            scene->unpause();
        }
        else {
            scene->standby();
        }

        Scene* nextScene = scene->start();

        if ( scene->getReset() ) {
            // リセット
            while ( !scenelist.empty() ) {
                scenelist.front()->destroy();
                scenelist.pop_front();
            }
            scenelist.push_front(initialScene);
        }
        else if ( nextScene == NULL ) {
            // 現在の場面を消す
            scene->destroy();
            scenelist.pop_front();
        }
        else if ( scene->getInterrupted() ) {
            // 現在の場面を残したまま、次の場面を挿入
            scene->pause();
            scenelist.push_front(nextScene);
        }
        else {
            // 現在の場面を消して、次の場面を挿入
            scene->destroy();
            scenelist.pop_front();
            scenelist.push_front(nextScene);
        }
        window->clearFaces();
    }
}

void Scenario::addScene(Scene* scene)
{
    scene->getMediator()->setWindow(window);
    scenelist.push_back(scene);
}

Scenario::~Scenario()
{
    common::freeList(scenelist);
    delete window;
}

