/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <math.h>

#include "common.h"
#include "CollisionGroup.h"
#include "CollisionListener.h"
#include "CollisionManager.h"
#include "List.h"

using namespace std;

CollisionManager::CollisionManager(SceneMediator* mediator)
{
    this->mediator = mediator;
}

void CollisionManager::standby()
{
    playerGroup = new List<CollisionListener>(10);
    playerShotGroup = new List<CollisionListener>(500);
    enemyGroup = new List<CollisionListener>(500);
    enemyShotGroup = new List<CollisionListener>(100);
    groundGroup = new List<CollisionListener>(10);
    groundShotGroup = new List<CollisionListener>(10);
    playerGroundGroup = new List<CollisionListener>(10);
    enemyGroundGroup = new List<CollisionListener>(10);
}

void CollisionManager::freeList(List<CollisionListener>* l)
{
    List<CollisionListener>::Item* i = l->getBegin();
    while ( i != l->getEnd() ) {
        i->getObject()->destroy(mediator);
        i = i->getNext();
    }
    delete l;
}

void CollisionManager::destroy()
{
    freeList(playerGroup);
    freeList(playerShotGroup);
    freeList(enemyGroup);
    freeList(enemyShotGroup);
    freeList(groundGroup);
    freeList(groundShotGroup);
    freeList(playerGroundGroup);
    freeList(enemyGroundGroup);
}
void recoil(CollisionListener* p, CollisionListener* e)
{
    double lx = (e->x + e->width/2) - (p->x + p->width / 2);
    double ly = (e->y + e->height/2) - (p->y + p->height / 2);

    if ( p->width > e->width ) {
        if ( p->x < e->x && e->x+e->width < p->x+p->width ) {
            if ( p->y < e->y && p->y+p->height < e->y+e->height ) {
                p->dy -= e->impactResilience / p->impactResilience;
                e->dy += p->impactResilience / e->impactResilience;
                return;
            }
            else if ( e->y < p->y && e->y+e->height < p->y+p->height ) {
                p->dy += e->impactResilience / p->impactResilience;
                e->dy -= p->impactResilience / e->impactResilience;
                return;
            }
        }
    }
    else {
        if ( e->x < p->x && p->x+p->width < e->x+e->width ) {
            if ( e->y < p->y && e->y+e->height < p->y+p->height ) {
                p->dy += e->impactResilience / p->impactResilience;
                e->dy -= p->impactResilience / e->impactResilience;
                return;
            }
            else if ( p->y < e->y && p->y+p->height < e->y+e->height ) {
                p->dy -= e->impactResilience / p->impactResilience;
                e->dy += p->impactResilience / e->impactResilience;
                return;
            }
        }
    }

    if ( p->height > e->height ) {
        if ( p->y < e->y && e->y+e->height < p->y+p->height ) {
            if ( p->x < e->x && p->x+p->width < e->x+e->width ) {
                p->dx -= e->impactResilience / p->impactResilience;
                e->dx += p->impactResilience / e->impactResilience;
                return;
            }
            else if ( e->x < p->x && e->x+e->width < p->x+p->width ) {
                p->dx += e->impactResilience / p->impactResilience;
                e->dx -= p->impactResilience / e->impactResilience;
                return;
            }
        }
    }
    else {
        if ( e->y < p->y && p->y+p->height < e->y+e->height ) {
            if ( e->x < p->x && e->x+e->width < p->x+p->width ) {
                p->dx += e->impactResilience / p->impactResilience;
                e->dx -= p->impactResilience / e->impactResilience;
                return;
            }
            else if ( p->x < e->x && p->x+p->width < e->x+e->width ) {
                p->dx -= e->impactResilience / p->impactResilience;
                e->dx += p->impactResilience / e->impactResilience;
                return;
            }
        }
    }

    double length = sqrt( lx*lx + ly*ly );
    p->dx -= ( lx / length ) * ( e->impactResilience / p->impactResilience );
    p->dy -= ( ly / length ) * ( e->impactResilience / p->impactResilience );
    e->dx += ( lx / length ) * ( p->impactResilience / e->impactResilience );
    e->dy += ( ly / length ) * ( p->impactResilience / e->impactResilience );
}

void CollisionManager::collisionDetection(List<CollisionListener>& list1, List<CollisionListener>& list2)
{
    List<CollisionListener>::Item* p;
    List<CollisionListener>::Item* e;
    bool living = true;
    for ( p = list1.getBegin(); p != list1.getEnd(); ) {
        CollisionListener* pfo = p->getObject();
        for ( e = list2.getBegin(); e != list2.getEnd(); ) {
            CollisionListener* efo = e->getObject();
            if ( efo->x <= pfo->x + pfo->width &&
                pfo->x <= efo->x + efo->width &&
                efo->y <= pfo->y + pfo->height &&
                pfo->y <= efo->y + efo->height )
            {
                living = pfo->collision(mediator, efo);

                if ( efo->collision(mediator, pfo) ) {

                    // 反動制御
                    if ( living ) {
                        recoil(pfo, efo);
                    }
                    e = e->getNext();
                }
                else { e = list2.erase(e); }

                if ( !living ) { break; }
            }
            else { e = e->getNext(); }
        }
        if ( living ) { p = p->getNext(); }
        else {
            p = list1.erase(p);
            living = true;
        }
    }
}

void CollisionManager::manage()
{
    collisionNow = true;

    collisionDetection(*playerGroup, *enemyGroup);
    collisionDetection(*playerGroup, *enemyShotGroup);
    collisionDetection(*playerGroup, *groundGroup);
    collisionDetection(*playerGroup, *groundShotGroup);
    collisionDetection(*playerGroup, *playerGroundGroup);
    collisionDetection(*playerShotGroup, *enemyGroup);
    collisionDetection(*playerShotGroup, *groundGroup);
    collisionDetection(*enemyGroup, *groundGroup);
    collisionDetection(*enemyGroup, *groundShotGroup);
    collisionDetection(*enemyGroup, *enemyGroundGroup);
    collisionDetection(*enemyShotGroup, *groundGroup);

    collisionNow = false;
}

void CollisionManager::addCollision(List<CollisionListener>* group, CollisionListener* listener)
{
    group->add(listener);
    listener->setListIndex(group->getEnd()->getPrevious());
}

void CollisionManager::addCollision(CollisionListener* listener)
{
    switch ( listener->getGroup() )
    {
        case CollisionGroup::PLAYER:
        {
            addCollision(playerGroup, listener);
            break;
        }
        case CollisionGroup::PLAYER_SHOT:
        {
            addCollision(playerShotGroup, listener);
            break;
        }
        case CollisionGroup::ENEMY:
        {
            addCollision(enemyGroup, listener);
            break;
        }
        case CollisionGroup::ENEMY_SHOT:
        {
            addCollision(enemyShotGroup, listener);
            break;
        }
        case CollisionGroup::GROUND:
        {
            addCollision(groundGroup, listener);
            break;
        }
        case CollisionGroup::GROUND_SHOT:
        {
            addCollision(groundShotGroup, listener);
            break;
        }
        case CollisionGroup::PLAYER_GROUND:
        {
            addCollision(playerGroundGroup, listener);
            break;
        }
        case CollisionGroup::ENEMY_GROUND:
        {
            addCollision(enemyGroundGroup, listener);
            break;
        }
        case CollisionGroup::NOCOLLISION:
        {
            /* do nothing; */
            break;
        }
        default:
        {
            cerr << "CollisionManager::addCollision(): Invalid group." << listener->getGroup()<< endl;
            exit(-1);
        }
    }
}

void CollisionManager::removeCollision(CollisionListener* listener)
{
    if ( collisionNow ) {
        cerr << "CollisionManager NOW." << endl;
        exit(-1);
    }

    List<CollisionListener>::Item* index = listener->getListIndex();
    switch ( listener->getGroup() )
    {
        case CollisionGroup::PLAYER: { playerGroup->erase(index); break; }
        case CollisionGroup::PLAYER_SHOT: { playerShotGroup->erase(index); break; }
        case CollisionGroup::ENEMY: { enemyGroup->erase(index); break; }
        case CollisionGroup::ENEMY_SHOT: { enemyShotGroup->erase(index); break; }
        case CollisionGroup::GROUND: { groundGroup->erase(index); break; }
        case CollisionGroup::GROUND_SHOT: { groundShotGroup->erase(index); break; }
        case CollisionGroup::PLAYER_GROUND: { playerGroundGroup->erase(index); break; }
        case CollisionGroup::ENEMY_GROUND: { enemyGroundGroup->erase(index); break; }
        case CollisionGroup::NOCOLLISION: { /* do nothing; */ break; }
        default:
        {
            cerr << "CollisionManager::removeCollision(): Invalid group." << endl;
            exit(-1);
        }
    }
} 

void CollisionManager::pause() { }
void CollisionManager::unpause() { }

int CollisionManager::getCollisionSize()
{
    return playerGroup->getSize()
        + enemyGroup->getSize()
        + groundGroup->getSize()
        + playerShotGroup->getSize()
        + enemyShotGroup->getSize()
        + groundShotGroup->getSize()
        + playerGroundGroup->getSize()
        + enemyGroundGroup->getSize()
        ;
}
List<CollisionListener>* CollisionManager::getGroupList(CollisionGroup::group group)
{
    switch ( group )
    {
        case CollisionGroup::PLAYER: { return playerGroup; }
        case CollisionGroup::PLAYER_SHOT: { return playerShotGroup; }
        case CollisionGroup::ENEMY: { return enemyGroup; }
        case CollisionGroup::ENEMY_SHOT: { return enemyShotGroup; }
        case CollisionGroup::GROUND: { return groundGroup; }
        case CollisionGroup::GROUND_SHOT: { return groundShotGroup; }
        case CollisionGroup::PLAYER_GROUND: { return groundShotGroup; }
        case CollisionGroup::ENEMY_GROUND: { return groundShotGroup; }
        default: { return NULL; }
    }
}

CollisionListener* CollisionManager::getListener(CollisionGroup::group group, int index)
{
    List<CollisionListener>* list = getGroupList(group);
    List<CollisionListener>::Item* item = list->getBegin();
    if ( list == NULL || list->isEmpty() ) { return NULL; }
    for ( int i = 0; i < index; i++ ) {
        item = item->getNext();
    }
    if ( item == list->getEnd() ) { item = item->getNext(); }
    return item->getObject();
}
