/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <SDL/SDL.h>
#include <iostream>

#include "KeyListener.h"
#include "InputManager.h"
#include "SceneMediator.h"

using namespace std;

class KeyMap
{
public:
    SDLKey keysym;
    KeyListener* listener;
    bool enabled;
};

InputManager::InputManager(SceneMediator* mediator)
{
    this->mediator = mediator;
}

void InputManager::addKeyListener(string keyName, KeyListener* listener)
{
    if ( keyMap.find(keyName) == keyMap.end() ) {
        keyMap[keyName] = new KeyMap();
        keyMap[keyName]->enabled = true;
        keyMap[keyName]->keysym = SDLK_UNKNOWN;
    }
    keyMap[keyName]->listener = listener;
}

void InputManager::removeKeyListener(string keyName)
{
    if ( keyMap.find(keyName) == keyMap.end() ) { return; }
    keyMap[keyName]->listener = NULL;
}

void InputManager::addKeyEvent(string keyName, SDLKey sym)
{
    if ( keyMap.find(keyName) == keyMap.end() ) {
        keyMap[keyName] = new KeyMap();
        keyMap[keyName]->enabled = true;
        keyMap[keyName]->listener = NULL;
    }
    keyMap[keyName]->keysym = sym;
}

void InputManager::setKeyEnabled(string keyName)
{
    if ( keyMap.find(keyName) == keyMap.end() ) {
        cerr << "無効なキーを有効にしようとしました("
        << keyName << ")" << endl;
    }
    keyMap[keyName]->enabled = true;
}

void InputManager::setKeyDisabled(string keyName)
{
    if ( keyMap.find(keyName) == keyMap.end() ) {
        cerr << "無効なキーを無効にしようとしました("
        << keyName << ")" << endl;
    }
    keyMap[keyName]->enabled = false;
}

void InputManager::removeKeyEvent(string keyName)
{
    keyMap[keyName]->keysym = SDLK_UNKNOWN;
}

void InputManager::saveKeyMap()
{
}

void InputManager::loadKeyMap()
{
}

void InputManager::manage()
{
    SDL_Event event;
    SDL_PollEvent(&event);
    switch(event.type) {
        case SDL_QUIT: { SDL_Quit(); exit(0); }
        case SDL_KEYDOWN:
        {
            switch (event.key.keysym.sym) {
                case SDLK_f: { mediator->toggleFullScreen(); break; }
                case SDLK_ESCAPE:
                case SDLK_q: { SDL_Quit(); exit(0); }
                case SDLK_r: { mediator->resetScene(); return; }
            }
        }
    }

    Uint8* keystate = SDL_GetKeyState(NULL);
    map<string, KeyMap*>::iterator i = keyMap.begin();
    map<string, KeyMap*>::iterator end = keyMap.end();

    while ( i != end ) {
        SDLKey keysym = (*i).second->keysym;
        KeyListener* listener = (*i).second->listener;

        if ( keysym == SDLK_UNKNOWN
            || listener == NULL
            || (*i).second->enabled == false )
        {
            i++;
            continue;
        }

        bool result;
        if ( keystate[keysym] ) {
            result = listener->keyPressed(mediator);
        }
        else {
            result = listener->keyReleased(mediator);
        }

        if ( !result ) {
            (*i).second->listener = NULL;
        }

        i++;
    }
}

int InputManager::getKeyMapSize()
{
    return keyMap.size();
}

int InputManager::getKeyListenerSize()
{
    return keyMap.size();
}

void InputManager::pause() {}
void InputManager::unpause() {}
void InputManager::standby() {}
void InputManager::destroy()
{
    map<string, KeyMap*>::iterator i = keyMap.begin();
    while ( i != keyMap.end() ) {
        delete (*i).second;
        i++;
    }
    keyMap.clear();
}
