/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <typeinfo>

#include "SceneMediator.h"
#include "ActionManager.h"
#include "CollisionManager.h"
#include "EventManager.h"
#include "FPSManager.h"
#include "InputManager.h"
#include "ScoreManager.h"
#include "WindowManager.h"
#include "KeyListener.h"

#include "../Scene.h"
#include "../Window.h"

SceneMediator::SceneMediator()
{
    actionManager = new ActionManager(this);
    collisionManager = new CollisionManager(this);
    eventManager = new EventManager(this);
    inputManager = new InputManager(this);
    scoreManager = new ScoreManager(this);
    windowManager = new WindowManager(this);
    fpsManager = new FPSManager(this);
    scene = new Scene(this);

    Manager** manager = new Manager*[8];
    manager[0] = windowManager;
    manager[1] = collisionManager;
    manager[2] = inputManager;
    manager[3] = actionManager;
    manager[4] = eventManager;
    manager[5] = scoreManager;
    manager[6] = fpsManager;
    manager[7] = NULL;
    scene->setManager(manager);
}

Scene* SceneMediator::getScene()
{
    return scene;
}

Scene* SceneMediator::makeScene()
{
    SceneMediator* mediator = new SceneMediator();
    return mediator->getScene();
}

void SceneMediator::addFace(Face* face)
{
    windowManager->addFace(face);
}

void SceneMediator::removeFace(Face* face)
{
    windowManager->removeFace(face);
}

int SceneMediator::getFaceSize()
{
    windowManager->getFaceSize();
}

void SceneMediator::addAction(ActionListener* listener)
{
    actionManager->addAction(listener);
}

void SceneMediator::removeAction(ActionListener* listener)
{
    actionManager->removeAction(listener);
}

int SceneMediator::getActionSize()
{
    return actionManager->getActionSize();
}

void SceneMediator::addCollision(CollisionListener* listener)
{
    collisionManager->addCollision(listener);
}

void SceneMediator::removeCollision(CollisionListener* listener)
{
    collisionManager->removeCollision(listener);
}
int SceneMediator::getCollisionSize()
{
    return collisionManager->getCollisionSize();
}

CollisionListener* SceneMediator::getObject(CollisionGroup::group group, int index)
{
    return collisionManager->getListener(group, index);
}

void SceneMediator::addKeyListener(string keyName, KeyListener* listener)
{
    inputManager->addKeyListener(keyName, listener);
}

void SceneMediator::addKeyListener(KeyListener* listener)
{
    inputManager->addKeyListener(listener->getKeyName(), listener);
    inputManager->addKeyEvent(listener->getKeyName(), listener->getKeySym());
}

void SceneMediator::removeKeyListener(string keyName)
{
    inputManager->removeKeyListener(keyName);
}

void SceneMediator::addKeyEvent(string keyName, SDLKey sym)
{
    inputManager->addKeyEvent(keyName, sym);
}

void SceneMediator::removeKeyEvent(string keyName)
{
    inputManager->removeKeyEvent(keyName);
}

void SceneMediator::setKeyEnabled(string keyName)
{
    inputManager->setKeyEnabled(keyName);
}

void SceneMediator::setKeyDisabled(string keyName)
{
    inputManager->setKeyDisabled(keyName);
}

int SceneMediator::getKeyMapSize()
{
    return inputManager->getKeyMapSize();
}

int SceneMediator::getKeyListenerSize()
{
    return inputManager->getKeyListenerSize();
}

void SceneMediator::addScore(int score)
{
    scoreManager->addScore(score);
}

int SceneMediator::getScore()
{
    return scoreManager->getScore();
}

void SceneMediator::setWindow(Window* window)
{
    windowManager->setWindow(window);
}
int SceneMediator::getWindowWidth()
{
    return windowManager->getWindow()->getWidth();
}

int SceneMediator::getWindowHeight()
{
    return windowManager->getWindow()->getHeight();
}

void SceneMediator::toggleFullScreen()
{
    windowManager->toggleFullScreen();
}

void SceneMediator::setEvent(list<SceneEvent*>* eventList)
{
    eventManager->setEvent(eventList);
}

void SceneMediator::endScene()
{
    this->scene->endScene();
}

void SceneMediator::gotoScene(Scene* scene)
{
    this->scene->gotoScene(scene);
}

void SceneMediator::gosubScene(Scene* scene)
{
    this->scene->gosubScene(scene);
}

void SceneMediator::resetScene()
{
    this->scene->resetScene();
}
