/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __SCENEMEDIATOR_H__INCLUDED__
#define __SCENEMEDIATOR_H__INCLUDED__

#include <list>
#include <SDL/SDL.h>
#include "CollisionGroup.h"

class ActionManager;
class CollisionManager;
class EventManager;
class FPSManager;
class InputManager;
class ScoreManager;
class WindowManager;

class ActionListener;
class CollisionListener;
class KeyListener;
class Face;
class Scene;
class SceneEvent;
class Window;

class SceneMediator
{
private:
    ActionManager* actionManager;
    CollisionManager* collisionManager;
    EventManager* eventManager;
    FPSManager* fpsManager;
    InputManager* inputManager;
    ScoreManager* scoreManager;
    WindowManager* windowManager;
    Scene* scene;
    
    SceneMediator();

public:
    static Scene* makeScene();
    Scene* getScene();
    
    void addFace(Face* face);
    void removeFace(Face* face);
    int getFaceSize();
    
    void addAction(ActionListener* listener);
    void removeAction(ActionListener* listener);
    int getActionSize();
    
    void addCollision(CollisionListener* listener);
    void removeCollision(CollisionListener* listener);
    CollisionListener* getObject(CollisionGroup::group group, int index);
    int getCollisionSize();
    
    void addKeyListener(std::string keyName, KeyListener* listener);
    void addKeyListener(KeyListener* listener);
    void removeKeyListener(std::string keyName);
    void addKeyEvent(std::string keyName, SDLKey sym);
    void removeKeyEvent(std::string keyName);
    void setKeyEnabled(std::string keyName);
    void setKeyDisabled(std::string keyName);
    int getKeyMapSize();
    int getKeyListenerSize();

    void addScore(int score);
    int getScore();

    void setWindow(Window* window);
    int getWindowWidth();
    int getWindowHeight();
    int getFacesSize();
    void toggleFullScreen();

    void setEvent(std::list<SceneEvent*>* eventList);

    void endScene();
    void gotoScene(Scene* scene);
    void gosubScene(Scene* scene);
    void resetScene();
};
#endif

