/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>

#include <SDL/SDL.h>

#include "common.h"
#include "Face.h"
#include "Library.h"
#include "List.h"
#include "Window.h"

using namespace std;

void Window::newFaces()
{
    faces = new List<Face>*[LAYERMAX];
    for ( int i = 0; i < LAYERMAX; i++ ) {
        if ( i == DEFAULTLAYER ) {
            faces[i] = new List<Face>(2000);
        }
        else {
            faces[i] = new List<Face>(100);
        }
    }
}
Window::Window()
{
    newFaces();
    surface = NULL;
    width = 320;
    height = 200;
    drawing = false;
}

Window::~Window()
{
    //common::freeList(faces);

    SDL_FreeSurface(surface);
}

void Window::setWidth(int width) { this->width = width; }
int Window::getWidth() { return width; }

void Window::setHeight(int height) { this->height = height; }
int Window::getHeight() { return height; }

void Window::setFPS(int fps) { this->fps = fps; }
int Window::getFPS() { return fps; }



void Window::showWindow()
{
    if ( surface != NULL ) {
        cout << "Window::showWindow(): Window is already showed." << endl;
        return;
    }
    surface = Library::createWindow(width, height);
}

void Window::draw()
{
    drawing = true;

    //backGroundColor = SDL_MapRGB(surface->format, 255,128,255);
    //SDL_FillRect(surface, NULL, backGroundColor);

    List<Face>::Item* f;
    for ( int i = 0; i < LAYERMAX; i++ ) {
        List<Face>* faces = this->faces[i];
        for ( f = faces->getBegin(); f != faces->getEnd(); ) {
            Face* face = f->getObject();
            if ( face->waitingForSurface ) {
                f = f->getNext();
            }
            else if ( face->destroy ) {
                delete (face);
                f = faces->erase(f);
            }
            else if ( face->surface == NULL || face->inactive ) {
                f = faces->erase(f);
            }
            else {
                face->nextFrame();
                SDL_SetAlpha(face->surface, face->alphaFlags, face->alpha);
                SDL_BlitSurface(face->surface, &(face->srcRect), surface, &(face->distRect));
                f = f->getNext();
            }
        }
    }
    SDL_UpdateRect(surface, 0, 0, 0, 0);
    drawing = false;
}

void Window::toggleFullScreen()
{
    surface = SDL_SetVideoMode(surface->w, surface->h, surface->format->BitsPerPixel, surface->flags ^ SDL_FULLSCREEN);
    //SDL_WM_ToggleFullScreen(surface);
}

void Window::addFace(Face* face)
{
    if ( face->layer >= LAYERMAX ) {
        cerr << "Window::addFace():Invalid Layer No." << endl;
        exit(-1);
    }
    faces[face->layer]->add(face);
    face->setListIndex(faces[face->layer]->getEnd()->getPrevious());
}

void Window::removeFace(Face* face)
{
    if(!drawing) {
        faces[face->layer]->erase(face->getListIndex());
    }
    else {
        cout << "Window::removeFace(): drawing now!" << endl;
    }
}
void Window::clearFaces() {
    for ( int i = 0; i < LAYERMAX; i++ ) {
        faces[i]->clear();
    }
}

List<Face>** Window::remakeFaces()
{
    List<Face>** oldfaces = faces;
    newFaces();
    return oldfaces;
}
void Window::setFaces(List<Face>** faces)
{
    clearFaces();
    this->faces = faces;
}

