/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <string>
#include <stdlib.h>
#include <string.h>

#include "../ActionSequence.h"

#include "ActionDestructionElement.h"
#include "ActionDirectionElement.h"
#include "ActionEndSceneElement.h"
#include "ActionHomingElement.h"
#include "ActionGenerateElement.h"
#include "ActionGosubSceneElement.h"
#include "ActionGotoSceneElement.h"
#include "ActionKeyEnabledElement.h"
#include "ActionKeyDisabledElement.h"
#include "ActionSequenceElement.h"
#include "ActionSlowdownRateElement.h"
#include "ActionTransparentElement.h"
#include "ActionTurnElement.h"
#include "ActionMoveElement.h"
#include "ActionMoveForwardElement.h"
#include "ActionMoveTargetElement.h"
#include "ActionNothingElement.h"
#include "ActionScoreElement.h"
#include "ActionSoundElement.h"

using namespace std;

ActionSequenceElement::ActionSequenceElement(char* super)
{
    name = "アクションシーケンス";
    superID = super;
    
    sequence = NULL;
    command = NULL;
    loopCount = NULL;
}

Element* ActionSequenceElement::getChild(char* name, char* super)
{
    if ( sequence == NULL ) { sequence = new list<ActionCommandElement*>; }

    if ( strcmp("アクションシーケンス", name) == 0 ) {
        ActionSequenceElement* com = new ActionSequenceElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("移動", name) == 0 ) {
        ActionMoveElement* com = new ActionMoveElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("場所移動", name) == 0 ) {
        ActionMoveTargetElement* com = new ActionMoveTargetElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("進路維持", name) == 0 ) {
        ActionMoveForwardElement* com = new ActionMoveForwardElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("進路変更", name) == 0 ) {
        ActionTurnElement* com = new ActionTurnElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("停止", name) == 0 ) {
        ActionNothingElement* com = new ActionNothingElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("発射", name) == 0 ) {
        ActionGenerateElement* com = new ActionGenerateElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("方向", name) == 0 ) {
        ActionDirectionElement* com = new ActionDirectionElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("自滅", name) == 0 ) {
        ActionDestructionElement* com = new ActionDestructionElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("減衰率", name) == 0 ) {
        ActionSlowdownRateElement* com = new ActionSlowdownRateElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("追尾", name) == 0 ) {
        ActionHomingElement* com = new ActionHomingElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("透過度", name) == 0 ) {
        ActionTransparentElement* com = new ActionTransparentElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("回数", name) == 0 ) {
        loopCount = new Element(super);
        return loopCount;
    }
    else if ( strcmp("効果音", name) == 0 ) {
        ActionSoundElement* com = new ActionSoundElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("スコア加算", name) == 0 ) {
        ActionScoreElement* com = new ActionScoreElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("場面移動", name) == 0 ) {
        ActionGotoSceneElement* com = new ActionGotoSceneElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("場面呼び出し", name) == 0 ) {
        ActionGosubSceneElement* com = new ActionGosubSceneElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("場面終わり", name) == 0 ) {
        ActionEndSceneElement* com = new ActionEndSceneElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("キー有効", name) == 0 ) {
        ActionKeyEnabledElement* com = new ActionKeyEnabledElement(super);
        sequence->push_back(com);
        return com;
    }
    else if ( strcmp("キー無効", name) == 0 ) {
        ActionKeyDisabledElement* com = new ActionKeyDisabledElement(super);
        sequence->push_back(com);
        return com;
    }
    else {
        invalidElement(name);
    }
}

void ActionSequenceElement::endElement()
{
}

int ActionSequenceElement::getLoopCount(std::list<std::string>* path)
{
    checkInheritance(getLoopCount, loopCount, ActionSequenceElement, 1);
    return atoi(loopCount->getValue(NULL).c_str());
}

list<ActionCommandElement*>*
ActionSequenceElement::getSequence(list<string>* path)
{
    checkInheritance(getSequence, sequence, ActionSequenceElement, new list<ActionCommandElement*>);
    return sequence;
}

ActionCommand* ActionSequenceElement::getCommand(std::list<std::string>* path)
{
    if ( command != NULL ) { return command; }
    ActionSequence* seq = ActionSequence::create();

    seq->setLoopCount( getLoopCount(NULL) );

    list<ActionCommandElement*>* sequence = getSequence(NULL);
    list<ActionCommandElement*>::iterator i = sequence->begin();
    while ( i != sequence->end() ) {
        seq->addCommand((*i)->getCommand(NULL)->clone());
        i++;
    }
    command = seq;
    return command;
}
