/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <list>
#include <string>
#include <iostream>
#include "Element.h"
using namespace std;

map<string, Element*> Element::namedElements;

Element::Element()
{
    this->name = "不明なエレメント";
    this->superID = "";
}

Element::Element(string name, string superID)
{
    this->name = name;
    this->superID = superID;
}

Element::Element(string superID)
{
    this->name = "不明なエレメント";
    this->superID = superID;
}

Element::~Element()
{
}

Element* Element::getChild(char* name, char* id, char* superID)
{
    Element* elm;
    if ( superID == NULL ) {
        elm = getChild(name, "");
    }
    else {
        elm = getChild(name, superID);
    }

    if ( id == NULL ) {
        /* do nothing */
    }
    else if ( namedElements.find(id) == namedElements.end() ) {
        namedElements[id] = elm;
        elm->id = id;
    }
    else {
        syntaxError((string)"<"+name+">が指定した名前("+id+")はすでに別のエレメントで使われています。");
    }
    return elm;
}

Element* Element::getChild(char* name, char* superID) {
    return new Element(name, superID);
}

void Element::endElement()
{
}

void Element::characters(char* value) {
    if ( value[0] != 0 ) {
        this->value = value;
    }
}


void Element::syntaxError(string message)
{
    cout << "行番号" << lineNum << "でエラーが発生しました: \"" << name << "." << id << "\": " << message << endl;
    throw "エラーによる中断";
    exit(-1);
}

void Element::invalidElement(char* name)
{
    cout << "指定されたタグ(" << name << ")は、この位置では使えません。" << endl;
    throw "エラーによる中断";
    exit(-1);
}

Element* Element::getParent()
{
    return parent;
}

void Element::setParent(Element* parent)
{
    this->parent = parent;
}

Element* Element::getSuper()
{
    if ( superID.empty() ) { return NULL; }
    if ( namedElements.find(superID) == namedElements.end() ) {
        syntaxError("無効な継承元("+superID+")が選択されています。");
        return NULL;
    }
    return namedElements[superID];
}

string Element::getValue(list<string>* path)
{
    path = checkInheritance_(path);
    if ( value.empty() ) {
        if ( getSuper() == NULL ) { return ""; }
        else { return getSuper()->getValue(path); }
    }
    else {
        return value;
    }
}

list<string>* Element::checkInheritance_(list<string>* path)
{
    if ( path == NULL ) { path = new list<string>; }
    list<string>::iterator i = path->begin();
    while ( i != path->end() ) {
        if ( (*i) == id ) {
            syntaxError("オブジェクトが循環継承しています。");
        }
        i++;
    }
    path->push_back(id);
    return path;
}

void Element::setLineNum(int lineNum)
{
    this->lineNum = lineNum;
}

void Element::setDir(boost::filesystem::path dir)
{
    this->dir = dir;
}
