/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __ELEMENT_H__INCLUDED__
#define __ELEMENT_H__INCLUDED__

#include <map>
#include <iostream>
#include <string>
#include <list>
#include <boost/filesystem/path.hpp>

/**
 * XML要素の基底クラス。
 *
 * 読み込まれたXMLのタグは対応する要素クラスによって解析される。
 */
class Element
{
protected:
    Element* parent;
    std::string superID;
    std::string name;
    std::string value;
    std::string id;
    int lineNum;
    boost::filesystem::path dir;

protected:
    void syntaxError(std::string message);
    void invalidElement(char* name);
    std::list<std::string>* checkInheritance_(std::list<std::string>* path);
    Element* getSuper();

public:
    /**
     * 名前付き要素のマップ
     *
     * 継承や参照などに使用する。
     */
    static std::map<std::string, Element*> namedElements;

    Element();
    Element(std::string name, std::string superID);
    Element(std::string superID);
    virtual ~Element();

    Element* getChild(char* name, char* id, char* superID);

    virtual Element* getChild(char* name, char* superID);
    virtual void endElement();
    virtual void characters(char* value);

    Element* getParent();
    void setParent(Element* parent);
    virtual std::string getValue(std::list<std::string>* path);
    void setLineNum(int lineNum);
    void setDir(boost::filesystem::path dir);

};

#define checkInheritance(getter, variable, type, default) if ( path == NULL ) {\
    path = new list<string>; }\
    list<string>::iterator i = path->begin();\
    while ( i != path->end() ) {\
        if ( (*i) == id ) {\
            syntaxError("オブジェクトが循環継承しています。");\
        }\
        i++;\
    }\
    path->push_back(id);\
    if ( variable == NULL ) {\
        if ( getSuper() == NULL ) { return default; }\
        else { return ((type*)getSuper())->getter(path); }\
    }

#define checkInheritanceNoDefault(getter, variable, type, error) if ( path == NULL ) {\
    path = new list<string>; }\
    list<string>::iterator i = path->begin();\
    while ( i != path->end() ) {\
        if ( (*i) == id ) {\
            syntaxError("オブジェクトが循環継承しています。");\
        }\
        i++;\
    }\
    path->push_back(id);\
    if ( variable == NULL ) {\
        if ( getSuper() == NULL ) { syntaxError(error); }\
        else { return ((type*)getSuper())->getter(path); }\
    }

#endif

