/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <string>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
using namespace std;

#include <SDL/SDL.h>

#include "../Event_AddFO.h"
#include "../Label.h"
#include "../Library.h"

#include "LabelElement.h"
#include "SceneEventElement.h"
#include "ColorElement.h"

LabelElement::LabelElement(char* super)
{
    name = "ラベル";
    superID = super;
    event = NULL;
    type = SCENEEVENT;
    text = NULL;
    size = NULL;
    image = NULL;
    x = NULL;
    y = NULL;
    dx = NULL;
    dy = NULL;
    frameCount = NULL;
    extraFrameCount = NULL;
    effect = NULL;
    color = NULL;
}

Element* LabelElement::getChild(char* name, char* super)
{
    if ( strcmp("文字", name) == 0 ) {
        text = new Element("文字", super);
        return text;
    }
    else if ( strcmp("画像", name) == 0 ) {
        image = new Element("画像", super);
        return image;
    }
    else if ( strcmp("大きさ", name) == 0 ) {
        size = new Element("大きさ", super);
        return size;
    }
    else if ( strcmp("横位置", name) == 0 ) {
        x = new Element("横位置", super);
        return x;
    }
    else if ( strcmp("縦位置", name) == 0 ) {
        y = new Element("縦位置", super);
        return y;
    }
    else if ( strcmp("横速度", name) == 0 ) {
        dx = new Element("横速度", super);
        return dx;
    }
    else if ( strcmp("縦速度", name) == 0 ) {
        dy = new Element("縦速度", super);
        return dy;
    }
    else if ( strcmp("時間", name) == 0 ) {
        frameCount = new Element("時間", super);
        return frameCount;
    }
    else if ( strcmp("予備時間", name) == 0 ) {
        extraFrameCount = new Element("予備時間", super);
        return extraFrameCount;
    }
    else if ( strcmp("演出", name) == 0 ) {
        effect = new Element("演出", super);
        return effect;
    }
    else if ( strcmp("色", name) == 0 ) {
        color = new ColorElement(super);
        return color;
    }
    else {
        invalidElement(name);
    }
}

void LabelElement::endElement()
{
}


const char* LabelElement::getText(list<string>* path)
{
    checkInheritance(getText, text, LabelElement, NULL);
    return text->getValue(NULL).c_str();
}
const char* LabelElement::getImage(list<string>* path)
{
    checkInheritance(getImage, image, LabelElement, NULL);
    return image->getValue(NULL).c_str();
}
double LabelElement::getX(list<string>* path)
{
    checkInheritance(getX, x, LabelElement, 0);
    return atof(x->getValue(NULL).c_str());
}
double LabelElement::getY(list<string>* path)
{
    checkInheritance(getY, y, LabelElement, 0);
    return atof(y->getValue(NULL).c_str());
}
double LabelElement::getDX(list<string>* path)
{
    checkInheritance(getDX, dx, LabelElement, 0);
    return atof(dx->getValue(NULL).c_str());
}
double LabelElement::getDY(list<string>* path)
{
    checkInheritance(getDY, dy, LabelElement, 0);
    return atof(dy->getValue(NULL).c_str());
}
int LabelElement::getFrameCount(list<string>* path)
{
    checkInheritance(getFrameCount, frameCount, LabelElement, INT_MAX);
    return atoi(frameCount->getValue(NULL).c_str());
}

int LabelElement::getExtraFrameCount(list<string>* path)
{
    checkInheritance(getExtraFrameCount, extraFrameCount, LabelElement, 0);
    return atoi(extraFrameCount->getValue(NULL).c_str());
}
int LabelElement::getEffect(list<string>* path)
{
    checkInheritance(getEffect, effect, LabelElement, 0);
    string e = effect->getValue(NULL);
    if ( e == "フェードイン" ) { return Label::FADEIN; }
    else if ( e == "フェードアウト" ) { return Label::FADEOUT; }
    else if ( e == "タイル" ) { return Label::TILE; }
    else if ( e == "無し" ) { return 0; }
    else { syntaxError("無効な演出"); }
}

SDL_Color* LabelElement::getColor(list<string>* path)
{
    checkInheritance(getColor, color, LabelElement, NULL);
    return color->getColor(NULL);
}

SceneEvent* LabelElement::getEvent()
{
    if ( event == NULL ) {
        if ( getText(NULL) != NULL ) {
            SDL_Color* color = getColor(NULL);
            if ( color == NULL ) {
                event = new Event_AddFO(
                    Label::createLabel(
                        getText(NULL),
                        getX(NULL),
                        getY(NULL),
                        getDX(NULL),
                        getDY(NULL),
                        getFrameCount(NULL),
                        getExtraFrameCount(NULL),
                        getEffect(NULL)
                    )
                );
            }
            else {
                event = new Event_AddFO(
                    Label::createLabel(
                        getText(NULL),
                        color,
                        getX(NULL),
                        getY(NULL),
                        getDX(NULL),
                        getDY(NULL),
                        getFrameCount(NULL),
                        getExtraFrameCount(NULL),
                        getEffect(NULL)
                    )
                );
            }
        }
        else if ( getImage(NULL) != NULL ) {
            event = new Event_AddFO(
                Label::createLabel(
                    Library::loadImage(getImage(NULL)),
                    getX(NULL),
                    getY(NULL),
                    getDX(NULL),
                    getDY(NULL),
                    getFrameCount(NULL),
                    getExtraFrameCount(NULL),
                    getEffect(NULL)
                )
            );
        }
        else {
            syntaxError("画像も文字も指定されていません。");
        }
    }
    return event;
}

