/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <string.h>

#include "../Library.h"
#include "../Scenario.h"

#include "ActionCommandElement.h"
#include "ActionMoveElement.h"
#include "ActionSequenceElement.h"
#include "ActionFOElement.h"
#include "BorderElement.h"
#include "IncludeElement.h"
#include "LabelElement.h"
#include "MusicElement.h"
#include "MusicStopElement.h"
#include "SceneElement.h"
#include "SceneTransitionElement.h"
#include "SoundElement.h"
#include "Window.h"

#include "RootElement.h"
#include "WindowElement.h"

using namespace std;

RootElement::RootElement(char* super)
{
    name = "シューティング";
    superID = super;
    windowElement = NULL;
}

Element* RootElement::getChild(char* name, char* super)
{
    if ( name == NULL ) {
        syntaxError("無効なエレメントです。");
    }
    else if ( strcmp("ウィンドウ", name) == 0 ) {
        if ( windowElement != NULL ) { delete windowElement; }
        windowElement = new WindowElement(super);
        return windowElement;
    }

    else if ( strcmp("場面", name) == 0 ) {
        SceneElement* se = new SceneElement(super);
        sceneElementList.push_back(se);
        return se;
    }
    else if ( strcmp("音楽", name) == 0 ) {
        return new MusicElement(super);
    }
    else if ( strcmp("音楽停止", name) == 0 ) {
        return new MusicStopElement(super);
    }
    else if ( strcmp("効果音", name) == 0 ) {
        return new SoundElement(super);
    }
    else if ( strcmp("ラベル", name) == 0 ) {
        return new LabelElement(super);
    }
    else if ( strcmp("場面遷移", name) == 0 ) {
        return new SceneTransitionElement(super);
    }
    else if ( strcmp("アクションシーケンス", name) == 0 ) {
        return new ActionSequenceElement(super);
    }
    else if ( strcmp("オブジェクト", name) == 0 ) {
        return new ActionFOElement(super);
    }
    else if ( strcmp("外部ファイル読み込み", name) == 0 ) {
        return new IncludeElement(super);
    }
    else if ( strcmp("境界線", name) == 0 ) {
        return new BorderElement(super);
    }
    else {
        invalidElement(name);
    }
}

void RootElement::endElement()
{
    Scenario* scenario = new Scenario();
    if ( windowElement == NULL ) { return; }
    scenario->setWindow( windowElement->getWindow(NULL) );

    if ( sceneElementList.empty() ) {
        syntaxError("最低でも一つ以上の場面が必要です。");
    }
    scenario->addScene(sceneElementList.back()->getScene(NULL));
    scenario->start();
}
