/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax/Locator.hpp>
#include <xercesc/framework/XMLRecognizer.hpp>

#include "Element.h"
#include "NoElement.h"
#include "SAXHandler.h"

using namespace std;

SAXHandler::SAXHandler()
{
    contextStack.push_back(new NoElement());
    XMLTransService::Codes resValue;
    transcoder = XMLPlatformUtils::fgTransService->makeNewTranscoderFor(
        XMLRecognizer::UTF_8, resValue, 1024, XMLPlatformUtils::fgMemoryManager
    );
    if ( resValue != XMLTransService::Ok ) {
        cout << "Invalid encoding name." << endl;
        exit(-1);
    }
}

XMLCh* SAXHandler::transcode(const char* s)
{
    if ( s == NULL ) { return NULL; }
    const XMLByte* src = (const XMLByte*)s;
    unsigned int srcCount = 0;
    XMLCh* toFill;
    unsigned int maxChars;
    unsigned int bytesEaten;
    unsigned char charSizes[4+1];
    while ( s[srcCount] != 0 ) { srcCount++; }
    toFill = new XMLCh[srcCount+1];
    maxChars = srcCount;
    int toFillCount = transcoder->transcodeFrom(src, srcCount, toFill, maxChars, bytesEaten, charSizes);
    toFill[toFillCount] = 0;
    return toFill;
}

char* SAXHandler::transcode(const XMLCh* s)
{
    if ( s == NULL ) { return NULL; }
    unsigned int srcCount = 0;
    XMLByte* toFill;
    unsigned int maxBytes;
    unsigned int charEaten;
    while ( s[srcCount] != 0 ) { srcCount++; }
    toFill = new XMLByte[srcCount*4+1];
    maxBytes = srcCount*4;
    int toFillCount = transcoder->transcodeTo(s, srcCount, toFill, maxBytes, charEaten, XMLTranscoder::UnRep_Throw );
    toFill[toFillCount] = 0;
    return (char*)toFill;
}

void SAXHandler::setDocumentLocator(const Locator* const locator)
{
    this->locator = locator;
}

char* SAXHandler::getValue(const Attributes& attrs, char* name) {
    XMLCh* attributeName = transcode(name);
    const XMLCh* encodedValue = attrs.getValue(attributeName);
    char* value = transcode(encodedValue);
    delete[] attributeName;
    return value;
}

void SAXHandler::startElement(
    const XMLCh* const uri,
    const XMLCh* const localname,
    const XMLCh* const qname,
    const Attributes& attrs)
{
    char* name = transcode(localname);

    Element* childelm;
    char* id = getValue(attrs, "名前");
    char* superID = getValue(attrs, "継承");

    try {
        childelm = contextStack.back()->getChild(name, id, superID);
        childelm->setParent(contextStack.back());
        childelm->setLineNum(locator->getLineNumber());
        childelm->setDir(dir);
    }
    catch(const char* msg) {
        cout << "ファイル：" << xmlfile << endl;
        cout << "行番号: " << locator->getLineNumber() << endl;
        cout << msg << endl;
        exit(-1);
    }

    contextStack.push_back(childelm);

    XMLString::release(&superID);
    XMLString::release(&id);
    delete[] name;
}

void SAXHandler::endElement(
    const XMLCh* const uri,
    const XMLCh* const localname,
    const XMLCh* const qname)
{
    try {
        contextStack.back()->endElement();
    }
    catch(const char* msg) {
        cout << "ファイル：" << xmlfile << endl;
        cout << "行番号: " << locator->getLineNumber() << endl;
        cout << msg << endl;
        exit(-1);
    }
    contextStack.pop_back();
}

void SAXHandler::characters
(
    const XMLCh* const chars,
    const unsigned int length
)
{
    XMLCh* chars_ = XMLString::replicate(chars);
    XMLString::trim(chars_);
    char* message = transcode(chars_);

    if ( message != NULL && message[0] != 0 ) {
        if ( contextStack.back() == NULL ) {
            cout << "致命的なエラー:" << endl;
            cout << "ファイル：" << xmlfile << endl;
            cout << "行番号: " << locator->getLineNumber() << endl;
            exit(-1);
        }
        contextStack.back()->characters(message);
    }

    XMLString::release(&chars_);
    delete[] message;
}

void SAXHandler::fatalError(const SAXParseException& exception)
{
    char* message = transcode(exception.getMessage());
    cout << "Fatal Error: "<< message << " "
        << xmlfile << " at line: " << exception.getLineNumber()
        << endl;
    delete[] message;
}

void SAXHandler::setDir(boost::filesystem::path dir)
{
    this->dir = dir;
}

void SAXHandler::setXMLFile(boost::filesystem::path xmlfile)
{
    this->xmlfile = xmlfile;
}
