/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include <stdlib.h>
#include <string.h>

#include "../Scene.h"
#include "../Scene/SceneMediator.h"
#include "../SceneEvent.h"
#include "../Event_AddFO.h"
#include "../Event_Sound.h"
#include "../Event_Music.h"
#include "../Event_MusicStop.h"
#include "../Event_Scene.h"

#include "ActionFOElement.h"
#include "BGScrollElement.h"
#include "BorderElement.h"
#include "LabelElement.h"
#include "MusicElement.h"
#include "MusicStopElement.h"
//#include "PlayerElement.h"
#include "SceneElement.h"
#include "SceneTransitionElement.h"
#include "SoundElement.h"
#include "WindowElement.h"

using namespace std;

SceneElement::SceneElement(char* super)
{
    name = "場面";
    superID = super;
    scene = NULL;
    elements = NULL;
}

Element* SceneElement::getChild(char* name, char* super)
{
    if ( elements == NULL ) {
        elements = new list<SceneEventElement*>;
    }
    if ( strcmp("ウィンドウ", name) == 0 ) {
        WindowElement* windowElement = new WindowElement(super);
        this->windowElement = windowElement;
        return windowElement;
    }
    if ( strcmp("ラベル", name) == 0 ) {
        LabelElement* label = new LabelElement(super);
        elements->push_back(label);
        return label;
    }
    else if ( strcmp("時間", name) == 0 ) {
        SceneEventElement* time = new SceneEventElement(super);
        time->setType(SceneEventElement::WAIT);
        elements->push_back(time);
        return time;
    }
//    else if ( strcmp("自機", name) == 0 ) {
//        PlayerElement* player = new PlayerElement(super);
//        elements->push_back(player);
//        return player;
//    }
    else if ( strcmp("効果音", name) == 0 ) {
        SoundElement* sound = new SoundElement(super);
        elements->push_back(sound);
        return sound;
    }
    else if ( strcmp("音楽", name) == 0 ) {
        MusicElement* music = new MusicElement(super);
        elements->push_back(music);
        return music;
    }
    else if ( strcmp("音楽停止", name) == 0 ) {
        MusicStopElement* music = new MusicStopElement(super);
        elements->push_back(music);
        return music;
    }
    else if ( strcmp("背景", name) == 0 ) {
        BGScrollElement* bgse = new BGScrollElement(super);
        elements->push_back(bgse);
        return bgse;
    }
    else if ( strcmp("場面遷移", name) == 0 ) {
        SceneTransitionElement* ste = new SceneTransitionElement(super);
        elements->push_back(ste);
        return ste;
    }
    else if ( strcmp("オブジェクト", name) == 0 ) {
        ActionFOElement* afoe = new ActionFOElement(super);
        elements->push_back(afoe);
        return afoe;
    }
    else if ( strcmp("境界線", name) == 0 ) {
        BorderElement* be = new BorderElement(super);
        elements->push_back(be);
        return be;
    }
    else {
        invalidElement(name);
    }
}

void SceneElement::endElement()
{
}


Scene* SceneElement::getScene(list<string>* path)
{
    if ( scene != NULL ) { return scene; }
    scene = SceneMediator::makeScene();

    list<SceneEventElement*>* elements = getElements(NULL);
    list<SceneEventElement*>::iterator i = elements->begin();
    list<SceneEvent*>* selist = new list<SceneEvent*>;
    int frameCount = 0;
    while ( i != elements->end() ) {
        switch ( (*i)->getType() ) {
            case SceneEventElement::SCENEEVENT:
            {
                SceneEvent* se = (*i)->getEvent();
                if ( se == NULL ) {
                    syntaxError("イベント生成ミスが発生しています。");
                }
                se->frameCount = frameCount;
                selist->push_back(se);
                break;
            }
            case SceneEventElement::WAIT:
            {
                frameCount += atoi((*i)->getValue(NULL).c_str());
                break;
            }
            default:
            {
                syntaxError("不明なイベントが指定されました。");
                break;
            }
        }
        i++;
    }

    scene->getMediator()->setEvent(selist);
    return scene;
}

WindowElement* SceneElement::getWindowElement()
{
    return windowElement;
}

list<SceneEventElement*>* SceneElement::getElements(list<string>* path)
{
    checkInheritance(getElements, elements, SceneElement, new list<SceneEventElement*>);
    return elements;
}
